/****************************************************************************
* abstract_config.cpp
*
* Platform independent wrapper for config files
*
* Copyright (C) 2004 Rafal Mantiuk <rafm@users.sourceforge.org>
* 'describeKey' function is derived from the OpieReader sources.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
****************************************************************************/

#include <stdlib.h>
#include <qtextcodec.h>

#include <stdio.h>

#include "abstract_config.h"

#ifdef DESKTOP
#include "registry.h"
#else
#include <qpe/config.h>
#endif

#ifdef DESKTOP

AbstractConfig::AbstractConfig( const char *name )
{
  registry = new Registry( getenv("HOME"), QString(".") + QString(name), "~" );
}

AbstractConfig::~AbstractConfig()
{
  delete registry;
}
  
void AbstractConfig::writeEntry ( const char *key, const char * value )
{
  registry->setString( key, value );
}

// Added specifically for capturing word=<utf8> in .zbedic
void AbstractConfig::writeEntry ( const char *key, const QString value )
{
  registry->setString( key, value.utf8() );
}

void AbstractConfig::writeEntry ( const char *key, bool value )
{
  registry->setBool( key, value );
}

void AbstractConfig::writeEntry ( const char *key, int num )
{
  registry->setInt( key, num );
}

QString AbstractConfig::readEntry( const char *key, const char *deflt ) const
{
  return QString::fromUtf8( registry->getString( key, deflt == NULL ? "" : deflt ) );
}

int AbstractConfig::readNumEntry( const char *key, int deflt ) const
{
  return registry->getInt( key, deflt );
}

bool AbstractConfig::readBoolEntry( const char *key, bool deflt ) const
{
  return registry->getBool( key, deflt );
}

QStringList AbstractConfig::readListEntry( const char *key, const char sep ) const
{
  QString entry = QString::fromUtf8( registry->getString( key ) );
  return escapeSafeSplit( sep, entry );
}

void AbstractConfig::writeEntry( const char *key, const QStringList &stringList, const char sep )
{
  char sep_str[2] = 
    {
      sep, 0 
    };
  registry->setString( key, stringList.join( sep_str ).utf8() );
}

void AbstractConfig::setGroup( const char *gname )
{
  //Do nothing - not supported
}

#else

AbstractConfig::AbstractConfig( const char *name )
{
  config = new Config( name );
}

AbstractConfig::~AbstractConfig()
{
  delete config;
}
  
void AbstractConfig::writeEntry ( const char *key, const char * value )
{
  config->writeEntry( key, value );
}

// Added specifically for capturing word=<utf8> in .zbedic
void AbstractConfig::writeEntry ( const char *key, const QString value )
{
  config->writeEntry( key, value );
}

void AbstractConfig::writeEntry ( const char *key, int num )
{
  config->writeEntry( key, num );
}

QString AbstractConfig::readEntry( const char *key, const char *deflt ) const
{
  return config->readEntry( key, deflt );
}

int AbstractConfig::readNumEntry( const char *key, int deflt ) const
{
  return config->readNumEntry( key, deflt );
}

bool AbstractConfig::readBoolEntry( const char *key, bool deflt ) const
{
  return config->readBoolEntry( key, deflt );
}

QStringList AbstractConfig::readListEntry( const char *key, const char sep ) const
{
  return escapeSafeSplit( sep, config->readEntry( key ) );
}

void AbstractConfig::writeEntry( const char *key, const QStringList &stringList, const char sep )
{
  config->writeEntry( key, stringList, sep );
}

void AbstractConfig::writeEntry ( const char *key, bool value )
{
  config->writeEntry( key, value );
}

void AbstractConfig::setGroup( const char *gname )
{
  config->setGroup( gname );
}




#endif


// Replace all occurences of "characters" in "str" with escape sequences
QString escapeString( QString str, QString characters )
{
  QString new_str = str;

  QString new_characters = QChar( '\\' ) + characters;
  
  const char *chrs = new_characters;
  while( chrs[0] != 0 ) {
    int ind = 0, new_ind;
    QString p_str = QString::null;

    while( true ) {
      new_ind = new_str.find( chrs[0], ind );
      if( new_ind == -1 ) {
        p_str += new_str.mid( ind );
        break;
      }
      int len = new_ind-ind;    
      p_str += new_str.mid( ind, len ) + "\\" + chrs[0];
      ind = new_ind+1;
    }

    new_str = p_str;
    
    chrs++;
  }

  return new_str;
  
}

// Replace all escape sequences with proper "characters" in "str"
QString unescapeString( QString str, QString characters )
{
  int ind = 0, new_ind;
  QString p_str = QString::null;

  while( true ) {
    new_ind = str.find( '\\', ind );
    if( new_ind == -1 || new_ind == ((int)str.length()-1) ) {
      p_str += str.mid( ind );
      break;
    }
    char next_ch = str[new_ind+1].latin1();
    int len = new_ind-ind;
    
    if( next_ch == '\\' || characters.find( next_ch ) != -1 ) {
      p_str += str.mid( ind, len ) + next_ch;
      ind = new_ind+2;
    } else {
      p_str += str.mid( ind, len+1 );
      ind = new_ind+1;
    }
  }

  return p_str;
  
}

QStringList escapeSafeSplit( const QChar & sep, const QString & str )
{
  QStringList str_list;

  int pos = 0, beg_pos = 0, new_pos;
  while( beg_pos < (int)str.length() ) {
    new_pos = str.find( sep, pos );
    if( new_pos == -1 ) {
      str_list.append( str.mid( beg_pos ) );
      break;
    }
    if( new_pos > 0 && str[new_pos-1] != '\\' ) {
      str_list.append( str.mid( beg_pos, new_pos-beg_pos ) );
      beg_pos = new_pos+1;
    }
    pos = new_pos+1;    
  }

  return str_list;
}
