#include "history_browser.h"

#include <qlabel.h>
#include <qlayout.h>
#include <qlistbox.h>
#include <qapplication.h>
#include <qframe.h>
#include <qpainter.h>

// =========== History Popup ========

struct TimeFrame
{
  const char *name;
  int days;
};

static const char* ESCAPE_CHARS = "@,";

static const TimeFrame time_frames[] = {
    { "Today (%s)", 0 },
    { "Yesterday (%s)", 1 },   
    { "Two days ago (%s)", 2 },
    { "Three days ago (%s)", 3 },
    { "Four days ago (%s)", 4 },
    { "Five days ago (%s)", 5 },
    { "Six days ago (%s)", 6 },
    { "One week ago", 7 },
    { "Two weeks ago", 14 },
    { "Thee weeks ago", 21 },
    { "Month ago", 30 },
    { "Older than a month", 60 
    }
    
};

HistoryPopup::HistoryPopup( QWidget *parent ) : QDialog( parent, NULL, TRUE, WType_Popup|WStyle_Tool )
{
  setMinimumWidth( 150 );
  setMinimumHeight( 200 );

  QBoxLayout *l = new QVBoxLayout( this );
  l->setMargin( 0 );
  
  hist_list = new QListBox( this );
  hist_list->setFrameStyle( QFrame::Box + QFrame::Plain );
  hist_list->setHScrollBarMode( QScrollView::AlwaysOff );

  l->addWidget( hist_list );

  connect( hist_list, SIGNAL( selected(int)), 
    this, SLOT(itemSelected(int)));
  connect( hist_list, SIGNAL( clicked(QListBoxItem*)), 
    this, SLOT(itemSelected(QListBoxItem*)));
  connect( hist_list, SIGNAL( returnPressed(QListBoxItem*)), 
    this, SLOT(itemSelected(QListBoxItem*)));

  selectedEntry = entries.end();
  
}

void HistoryPopup::addEntry( const QString &entry, const QString &dict )
{
  // Skip if the word is in Today's history
  HistoryList::iterator it;
  for( it = entries.begin(); it != entries.end() && it->date.daysTo( QDate::currentDate() ) == 0; it++ )
    if( it->entry == entry && it->dict == dict )
      return;
  
  entries.push_front( HistoryEntry( entry, dict, QDate::currentDate() ) );
  selectedEntry = entries.begin();

  // Make sure that the history does not grow too large
  if( entries.size() > 100 ) {
    entries.pop_back();
  }
  
    
}

void HistoryPopup::updateEntries()
{
  hist_list->clear();

  small_font = new QFont( font() );
  if( small_font->pointSize() > 8 )
    small_font->setPointSize( small_font->pointSize() - 4 );
    
  int cur_time_frame = 0;
  const int time_frame_count = sizeof( time_frames )/sizeof( TimeFrame );
  HistoryList::iterator it;
  for( it = entries.begin(); it != entries.end(); it++ ) {
    bool new_timeframe = false;
    while( cur_time_frame < time_frame_count &&
        it->date.daysTo( QDate::currentDate() ) >= time_frames[cur_time_frame].days
        ) {
        cur_time_frame++;
        new_timeframe = true;
    }
    if( new_timeframe ) {
//      printf( "days: %d\n", it->date.daysTo( QDate::currentDate() ) );

      QString label;
      label.sprintf( time_frames[cur_time_frame-1].name, (const char*)it->date.dayName( it->date.dayOfWeek() ) );
      hist_list->insertItem(new ListBoxSeparator( label, small_font ));
    }
    HistoryLBEntry *new_item = new HistoryLBEntry( it->entry );
    hist_list->insertItem(new_item);
    it->item_index = hist_list->index(new_item);
    if( it == selectedEntry )
      hist_list->setCurrentItem( it->item_index );
  }  
}

void HistoryPopup::polish()
{
//  printf( "polish size: %d\n", small_font->pixelSize() );
}


void HistoryPopup::show()
{
  
  QDialog::show();

  
  hist_list->centerCurrentItem();  
}

void HistoryPopup::centerCurrentItem()
{
  hist_list->centerCurrentItem();
  hist_list->setContentsPos( hist_list->contentsX(), hist_list->contentsY()+40 );
  
}

void HistoryPopup::clearHistory()
{
  entries.clear();
  selectedEntry=entries.end();
}

ListBoxSeparator::ListBoxSeparator( const QString &text, QFont *font, QListBox * listbox ) :
  QListBoxItem( listbox ), text( text ), font( font )
{
  setSelectable( FALSE );
  QFontMetrics fm( *font );  
  sep_height = fm.height();
}

int ListBoxSeparator::height( const QListBox * ) const
{
  return sep_height;
}

int ListBoxSeparator::width( const QListBox *lb ) const
{
  return lb->visibleWidth();
}

void ListBoxSeparator::paint( QPainter *painter )
{
//  const int min_margin = 10;

//  printf( "Width: %d\n", width(listBox()) );

//  sep_height = font->pixelSize() + 2;
  int vis_width = listBox()->visibleWidth();

  QFontMetrics fm( *font );
  int text_width = fm.width( text );
  int side_bars = (vis_width - text_width)/2 - 6;

  if( side_bars > 0 ) {
    painter->drawLine( 0, sep_height/2, side_bars, sep_height/2 );
    painter->drawLine( vis_width-side_bars, sep_height/2, vis_width, sep_height/2 );
  }
  
  painter->setFont( *font );
  painter->setBackgroundColor( QColor( 255, 255, 255 ) );
  painter->setBackgroundMode ( Qt::OpaqueMode );  
  painter->drawText( 0, 0, vis_width, sep_height, Qt::AlignCenter, text );
  
}


void HistoryPopup::loadHistory( AbstractConfig &conf )
{
  QStringList history = conf.readListEntry("history", ',');
  for(unsigned int i = 0; i < history.count(); i++) {
    QStringList hentry = escapeSafeSplit( '@', history[i] );
    
//    int atSignPos = history[i].find( '@' );
//    int hashSignPos = -1;
//    if( atSignPos != -1 )
//      int hashSignPos = history[i].find( '@', atSignPos );
//    QString entry = history[i].left( atSignPos );
//    QString dict = history[i].mid( atSignPos+1, hashSignPos-atSignPos-1 );

//    printf( "count = %d\n", hentry.count() );
    QString entry = unescapeString( hentry[0], ESCAPE_CHARS );
    QString dict = hentry.count()>1 ? unescapeString( hentry[1], ESCAPE_CHARS ) :
      QString::null;
    QDate date = hentry.count()==5 ?
      QDate( hentry[2].toInt(), hentry[3].toInt(), hentry[4].toInt() ) :
      QDate( 2000, 1, 1 );

//    printf( "%d %d %d %s\n", hentry[2].toInt(), hentry[3].toInt(), hentry[4].toInt(), (const char*)hentry[2] );
    entries.push_back( HistoryEntry( entry, dict, date ) );
  }
  int historyCurrent = conf.readNumEntry( "history-current", -1 );
  hist_list->setCurrentItem( historyCurrent );
}


void HistoryPopup::saveHistory( AbstractConfig &conf )
{
  QStringList history;
  
  HistoryList::iterator it;
  for( it = entries.begin(); it != entries.end(); it++ ) {
    history.append( escapeString( it->entry, ESCAPE_CHARS ) + "@" +
      escapeString( it->dict, ESCAPE_CHARS ) + "@" +
      QString::number( it->date.year() ) + "@" +
      QString::number( it->date.month() ) + "@" +
      QString::number( it->date.day() ) );
  }
  conf.writeEntry("history", history, ',');
  conf.writeEntry("history-current", hist_list->currentItem() );

}

void HistoryPopup::itemSelected( QListBoxItem * item )
{
  int index = hist_list->index( item );
  itemSelected( index );
}

void HistoryPopup::itemSelected( int item )
{
  HistoryList::iterator it;
  for( it = entries.begin(); it != entries.end(); it++ ) {
    if( it->item_index == item ) break;
  }

  if( it == entries.end() ) return;

  selectedEntry = it;

//  printf( "entry: %s\n", (const char*)(it->entry) );

  accept();
  
}


// =========== Browser Stack ===========

BrowserStack::BrowserStack()
{
  current = stack.end();
}


void BrowserStack::addEntry( const BrowserStackEntry &entry )
{
  // Do not add if the same entry has been recently added
 if( current != stack.begin() ) {
    BrowserStackList::iterator previous = current;
    previous--;
    if( previous->entry == entry.entry &&
      previous->dict == entry.dict )
      return;
  }

  // Do not add if it is the next entry
//   if( current != stack.end() ) {
//     if( current->entry == entry.entry &&
//       current->dict == entry.dict )
//       return;
//   }

//  printf( "Entry added: %s\n", (const char*)entry.entry );
//  BrowserStackList::iterator it;  
  
  stack.insert( current, entry );
  stack.erase( current, stack.end() );
  current = stack.end();

  // Make sure that the stack does not grow too large
  if( stack.size() > 30 )
    stack.pop_front();

//   printf( "  Entries:" );
//   for( it = stack.begin(); it != stack.end(); it++ )
//     printf( " %s,", (const char*)it->entry );
//   printf( "\n" );
  
}

const BrowserStackEntry &BrowserStack::backEntry()
{
  if( !hasBack() )
    return nullEntry;
  return *(--current);
}

const BrowserStackEntry &BrowserStack::forwardEntry()
{
  if( !hasForward() )
    return nullEntry;

//  printf( "next: %s\n", (const char*)current->entry );

  return *(++current);
}

bool BrowserStack::hasBack()
{
  return current != stack.begin();
}

bool BrowserStack::hasForward()
{
  if( current == stack.end() )
    return false;
  BrowserStackList::iterator next = current;
  next++;
  return next != stack.end();
}


void BrowserStack::save( AbstractConfig &conf )
{
  QStringList bstack;
  
  BrowserStackList::iterator it;
  int current_no = -1;
  int no = 0;
  for( it = stack.begin(); it != stack.end(); it++ ) {
    bstack.append( escapeString( it->entry, ESCAPE_CHARS ) + "@" +
      escapeString( it->dict, ESCAPE_CHARS ) + "@" +
      QString::number( it->pos ) );
    if( it == current )
      current_no = no;
    no++;
  }
  conf.writeEntry("browser-stack", bstack, ',');
  conf.writeEntry("browser-current", current_no );
}

void BrowserStack::load( AbstractConfig &conf )
{
  QStringList bstack = conf.readListEntry("browser-stack", ',');
  int current_no = conf.readNumEntry( "browser-current", -1 );
  if( current_no == -1 ) current = stack.end();
  
  int no = 0;
  for(unsigned int i = 0; i < bstack.count(); i++) {
    QStringList bentry = escapeSafeSplit( '@', bstack[i] );
    
    QString entry = unescapeString( bentry[0], ESCAPE_CHARS );
    QString dict = bentry.count()>1 ?
      unescapeString( bentry[1], ESCAPE_CHARS ) : QString::null;
    int pos = (bentry.count()==2 ? bentry[3].toInt() : 0 );

    stack.push_back( BrowserStackEntry( entry, dict, pos ) );

    if( no == current_no ) {
      current = stack.end();
      current--;
    }
    no++;
  }
}
